@@ -22,7 +22,7 @@
 ***********************************************************************/
 
 #include "env.h"
-#include "glpspx.h"
+#include "simplex.h"
 #include "spxat.h"
 #include "spxnt.h"
 #include "spxchuzc.h"
@@ -30,16 +30,16 @@
 #include "spxprob.h"
 
 #define USE_AT 0
-/* 0 - use A in row-wise format
- * 1 - use N in row-wise format */
+/* 1 - use A in row-wise format
+ * 0 - use N in row-wise format */
 
 #define EXCL 1
-/* 0 - don't exclude variables
- * 1 - exclude fixed non-basic variables */
+/* 1 - exclude fixed non-basic variables
+ * 0 - don't exclude variables */
 
 #define SHIFT 1
-/* 0 - don't shift bounds of variables
-   1 - shift bounds of variables toward zero */
+/* 1 - shift bounds of variables toward zero
+ * 0 - don't shift bounds of variables */
 
 #define CHECK_ACCURACY 0
 /* (for debugging) */
@@ -54,7 +54,7 @@ struct csa
        * +1 - minimization
        * -1 - maximization */
       double *c; /* double c[1+n]; */
-      /* copy of objective coefficients */
+      /* copy of original objective coefficients */
       SPXAT *at;
       /* mxn-matrix A of constraint coefficients, in sparse row-wise
        * format (NULL if not used) */
@@ -206,7 +206,7 @@ static int set_penalty(struct csa *csa, double tol, double tol1)
 }
 
 /***********************************************************************
-*  check_feas - check feasibility of basic solution
+*  check_feas - check primal feasibility of basic solution
 *
 *  This routine checks if the specified values of all basic variables
 *  beta = (beta[i]) are within their bounds.
@@ -490,10 +490,13 @@ static void check_accuracy(struct csa *csa)
 *  because the list is assumed to be non-empty. Then the routine
 *  computes q-th column T[*,q] of the simplex table T[i,j] and chooses
 *  basic variable xB[p]. If the pivot T[p,q] is small in magnitude,
-*  the routine tries to choose another xN[q] and xB[p] in order to
+*  the routine attempts to choose another xN[q] and xB[p] in order to
 *  avoid badly conditioned adjacent bases. */
 
-static void choose_pivot(struct csa *csa)
+#if 1 /* 17/III-2016 */
+#define MIN_RATIO 0.0001
+
+static int choose_pivot(struct csa *csa)
 {     SPXLP *lp = csa->lp;
       int m = lp->m;
       int n = lp->n;
@@ -501,12 +504,17 @@ static void choose_pivot(struct csa *csa)
       double *d = csa->d;
       SPXSE *se = csa->se;
       int *list = csa->list;
-      int nnn, try, q, t, p_flag, p;
       double *tcol = csa->work;
-      /* initial number of eligible non-basic variables */
+      double tol_piv = csa->tol_piv;
+      int try, nnn, /*i,*/ p, p_flag, q, t;
+      double big, /*temp,*/ best_ratio;
+      xassert(csa->beta_st);
+      xassert(csa->d_st);
+more: /* initial number of eligible non-basic variables */
       nnn = csa->num;
       /* nothing has been chosen so far */
       csa->q = 0;
+      best_ratio = 0.0;
       try = 0;
 try:  /* choose non-basic variable xN[q] */
       xassert(nnn > 0);
@@ -522,36 +530,55 @@ try:  /* choose non-basic variable xN[q] */
       xassert(1 <= q && q <= n-m);
       /* compute q-th column of the simplex table */
       spx_eval_tcol(lp, q, tcol);
+#if 0
+      /* big := max(1, |tcol[1]|, ..., |tcol[m]|) */
+      big = 1.0;
+      for (i = 1; i <= m; i++)
+      {  temp = tcol[i];
+         if (temp < 0.0)
+            temp = - temp;
+         if (big < temp)
+            big = temp;
+      }
+#else
+      /* this still puzzles me */
+      big = 1.0;
+#endif
       /* choose basic variable xB[p] */
       if (!csa->harris)
       {  /* textbook ratio test */
          p = spx_chuzr_std(lp, csa->phase, beta, q,
-            d[q] < 0.0 ? +1. : -1., tcol, &p_flag, csa->tol_piv,
+            d[q] < 0.0 ? +1. : -1., tcol, &p_flag, tol_piv,
             .30 * csa->tol_bnd, .30 * csa->tol_bnd1);
       }
       else
       {  /* Harris' two-pass ratio test */
          p = spx_chuzr_harris(lp, csa->phase, beta, q,
-            d[q] < 0.0 ? +1. : -1., tcol, &p_flag , csa->tol_piv,
+            d[q] < 0.0 ? +1. : -1., tcol, &p_flag , tol_piv,
             .50 * csa->tol_bnd, .50 * csa->tol_bnd1);
       }
+      if (p <= 0)
+      {  /* primal unboundedness or special case */
+         csa->q = q;
+         memcpy(&csa->tcol[1], &tcol[1], m * sizeof(double));
+         csa->p = p;
+         csa->p_flag = p_flag;
+         best_ratio = 1.0;
+         goto done;
+      }
       /* either keep previous choice or accept new choice depending on
        * which one is better */
-      if (csa->q == 0 || p <= 0 ||
-         fabs(tcol[p]) > fabs(csa->tcol[csa->p]))
+      if (best_ratio < fabs(tcol[p]) / big)
       {  csa->q = q;
          memcpy(&csa->tcol[1], &tcol[1], m * sizeof(double));
          csa->p = p;
          csa->p_flag = p_flag;
+         best_ratio = fabs(tcol[p]) / big;
       }
-      /* check if current choice is acceptable */
-      if (p <= 0 || fabs(csa->tcol[csa->p]) >= 0.001)
+      /* check if the current choice is acceptable */
+      if (best_ratio >= MIN_RATIO || nnn == 1 || try == 5)
          goto done;
-      if (nnn == 1)
-         goto done;
-      if (try == 5)
-         goto done;
-      /* try to choose another xN[q] and xB[p] */
+      /* try to choose other xN[q] and xB[p] */
       /* find xN[q] in the list */
       for (t = 1; t <= nnn; t++)
          if (list[t] == q) break;
@@ -563,8 +590,21 @@ try:  /* choose non-basic variable xN[q] */
       /* repeat the choice */
       goto try;
 done: /* the choice has been made */
-      return;
+#if 1 /* FIXME: currently just to avoid badly conditioned basis */
+      if (best_ratio < .001 * MIN_RATIO)
+      {  /* looks like this helps */
+         if (bfd_get_count(lp->bfd) > 0)
+            return -1;
+         /* didn't help; last chance to improve the choice */
+         if (tol_piv == csa->tol_piv)
+         {  tol_piv *= 1000.;
+            goto more;
+         }
+      }
+#endif
+      return 0;
 }
+#endif
 
 /***********************************************************************
 *  sum_infeas - compute sum of primal infeasibilities
@@ -657,7 +697,7 @@ skip: return;
 *
 *  This routine is a driver to the two-phase primal simplex method.
 *
-*  This routine returns one of the following codes:
+*  On exit this routine returns one of the following codes:
 *
 *  0  LP instance has been successfully solved.
 *
@@ -718,7 +758,7 @@ loop: /* main loop starts here */
             goto fini;
          }
          if (cond > 0.001 / DBL_EPSILON)
-         {  if (msg_lev >= GLP_MSG_ALL)
+         {  if (msg_lev >= GLP_MSG_ERR)
                xprintf("Warning: basis matrix is ill-conditioned (cond "
                   "= %.3g)\n", cond);
          }
@@ -871,7 +911,14 @@ loop: /* main loop starts here */
          }
       }
       /* choose xN[q] and xB[p] */
+#if 0 /* 17/III-2016 */
       choose_pivot(csa);
+#else
+      if (choose_pivot(csa) < 0)
+      {  lp->valid = 0;
+         goto loop;
+      }
+#endif
       /* check for unboundedness */
       if (csa->p == 0)
       {  if (csa->beta_st != 1)
@@ -916,9 +963,10 @@ loop: /* main loop starts here */
       else
          spx_nt_prod(lp, nt, trow, 1, -1.0, rho);
       /* FIXME: tcol[p] and trow[q] should be close to each other */
+      xassert(trow[csa->q] != 0.0);
       /* update reduced costs of non-basic variables for adjacent
        * basis */
-      if (spx_update_d(lp, d, csa->p, csa->q, trow, tcol) <= 1e-8)
+      if (spx_update_d(lp, d, csa->p, csa->q, trow, tcol) <= 1e-9)
       {  /* successful updating */
          csa->d_st = 2;
          if (csa->phase == 1)
@@ -984,7 +1032,7 @@ int spx_primal(glp_prob *P, const glp_smcp *parm)
 {     /* driver to primal simplex method */
       struct csa csa_, *csa = &csa_;
       SPXLP lp;
-#ifdef USE_AT
+#if USE_AT
       SPXAT at;
 #else
       SPXNT nt;
@@ -1009,10 +1057,9 @@ int spx_primal(glp_prob *P, const glp_smcp *parm)
          default:
             xassert(P != P);
       }
-      csa->dir = (P->dir == GLP_MIN ? +1 : -1);
       csa->c = talloc(1+csa->lp->n, double);
       memcpy(csa->c, csa->lp->c, (1+csa->lp->n) * sizeof(double));
-#ifdef USE_AT
+#if USE_AT
       /* build matrix A in row-wise format */
       csa->at = &at;
       csa->nt = NULL;
@@ -1054,6 +1101,12 @@ int spx_primal(glp_prob *P, const glp_smcp *parm)
             csa->harris = 0;
             break;
          case GLP_RT_HAR:
+#if 1 /* 16/III-2016 */
+         case GLP_RT_FLIP:
+            /* FIXME */
+            /* currently for primal simplex GLP_RT_FLIP is equivalent
+             * to GLP_RT_HAR */
+#endif
             csa->harris = 1;
             break;
          default:
